;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo-cm
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(unless (find-package :compo-cm) (make-package :compo-cm))
(in-package :compo-cm)
(use-package :compo)

(when (or *reload-all*
          (not (find :note *features*)) )
  (load (make-pathname :host *host*
                       :device *device* 
                       :directory (append *compo-root* '("note"))
                       :name "package" 
                       :type *source-type*) ) )

(use-package :note)

(export '(*compo-cmn-source* 
          *compo-cmn-object*

          *default-mode*
          *default-key*
          *default-time*
          *default-output-file*
          *default-size*
          *default-free-expansion-factor*
          *default-staff-separation*
          *default-line-separation*

          score
          cmnize
          init-cleaves
          cleave-change ))

(proclaim '(special
            *default-mode*
            *default-key*
            *default-time*
            *default-output-file*
            *default-size*
            *default-free-expansion-factor* ))

(setf (logical-pathname-translations "cmn")
      `(("**;*.*" , (make-pathname :host *host*
                     :device *device*
                     :directory `(,@*compo-root* "compo-cm" "cmn" :wild-inferiors)
                     :name :wild
                     :type :wild ))) )

(defvar *default-mode* :major-mode

"Variable
   *default-mode*
Purpose
   The default mode used in the drawing of score is the major mode." )

(defvar *default-key* :C

"Variable
   *default-key*
Purpose
   The default key used in the drawing of score is the key C." )

(defvar *default-time* 'cmn::common-time

"Variable
   *default-time*
Purpose
   The default time used in the drawing of score is the common time." )

(defparameter *default-output-file*
  (make-pathname :host *host*
                 :device *device*
                 :directory *compo-root*
                 :name "cmn-1" 
                 :type "eps")

"Parameter
   *default-output-file*
Purpose
   The default output file used in the drawing of score is 'cmn-1.eps' in the compo root
   directory." )

(defvar *default-size* 28

"Variable
   *default-size*
Purpose
   The default size used in the drawing of score is 28." )

(defvar *default-free-expansion-factor* 2

"Variable
   *default-free-expansion-factor*
Purpose
   The default free expansion factor used in the drawing of score is 2." )

(defvar *default-staff-separation* 1.5

"Variable
   *default-staff-separation*
Purpose
   The default staff separation value used in the drawing of score is 1.5." )

(defvar *default-line-separation* 2

"Variable
   *default-line-separation*
Purpose
   The default line separation value used in the drawing of score is 2." )

(defvar *voices*
  '(cmn::treble cmn::soprano cmn::mezzo-soprano cmn::alto
    cmn::tenor cmn::baritone cmn::bass cmn::sub-bass
    cmn::double-bass cmn::percussion cmn::treble cmn::treble
    cmn::treble cmn::treble cmn::treble cmn::french-violin ) )

(defvar *staves* nil)

(make-and-load "cmn:bin;score"
               "cmn:src;score.lisp" )

(make-and-load "cmn:bin;cmnize"
               "cmn:src;cmnize.lisp" )

(make-and-load "cmn:bin;grammar"
               "cmn:src;grammar.lisp" )

(make-and-load "cmn:bin;cleave-change"
               "cmn:src;cleave-change.lisp" )

(make-and-load "cmn:bin;cleave-change-help"
               "cmn:src;sugar;cleave-change-help.lisp" )

(make-and-load "cmn:bin;specialized-help"
               "cmn:src;sugar;specialized-help.lisp" )

(make-and-load "cmn:bin;check-cleave-change"
               "cmn:src;sugar;error;check-cleave-change.lisp" )

(pushnew :compo-cmn *features*)