;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo-cm
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo-cm)
    
(defun score (note
              &key (mode *default-mode*)
              (key *default-key*)
              (time *default-time*)
              (output-type cmn::*cmn-output-type*)
              (output-file *default-output-file*)
              (size *default-size*)
              (free-expansion-factor *default-free-expansion-factor*)
              (staff-separation *default-staff-separation*)
              (line-separation *default-line-separation*)
              (initial-onset 0)
              tied-voices )

"Function
   score (note &key :mode
                    :key
                    :time
                    :output-type
                    :output-file
                    :size
                    :free-expansion-factor
                    :staff-separation
                    :line-separation
                    :initial-onset
                    :tied-voices)
Arguments
   A note structure.
   A mode being one of the two keywords :major-mode or :minor-mode, defaulting to
   the current value of the special variable *default-mode*.
   A key value among the keywords :C :D :E :F :G :A :B :CS :DS :FS :GS :AS :CF :DF :EF
   :GF :AF and :BF, defaulting to the current value of the special variable
   *default-key*.
   A time being either :
   . A number : in this case, the time signature is considered to be number/4
     (for example 3/4 if the number is 3);
   . A symbol which can be 'cmn::alla-breve', 'cmn::common-time', 'cmn::cut-time' or
     'cmn::unmetered';
   . A list of two numbers (for example '(3 8)' for a time signature of 3/8.
   The time defaults to the current value of the special variable *default-time*.
   An output type which schould complies to those of the package :cmn (see Common Music
   Notation from the CCRMA), and defaulting to the current value of the special variable
   'cmn::*cmn-output-type*'.
   An output file being a string or a pathname designing a file to be produced,
   defaulting to the current value of the special variable *default-output-file*.
   A size defaulting to the current value of the special variable *default-size* (see
   Common Music Notation).
   A free expansion factor defaulting to the current value of the special variable
   *default-free-expansion-factor* (see Common Music Notation).
   A staff separation value defaulting to the current value of the special variable
   *default-staff-separation* (see Common Music Notation).
   A line separation value defaulting to the current value of the special variable
   *default-line-separation* (see Common Music Notation).
   An initial onset defaulting to 0.
   The list of the voice classes to be tied defaulting to the empty list.
Values
   No value is returned.
   As a side effect, The Common Music Notation form obtained by applying 'cmnize' to
   the given note, once normalized (realized with the context 'standard-context') with
   the received key arguments is evalued, in order to have the corresponding score
   drawn."

  (eval (cmnize (realize note :preserve-lyrics t)
                :mode mode
                :key key
                :time time
                :output-type output-type
                :output-file output-file
                :size size
                :free-expansion-factor free-expansion-factor
                :staff-separation staff-separation
                :line-separation line-separation
                :initial-onset initial-onset
                :tied-voices tied-voices )) )