;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defclass atomization (context-item)()
  (:documentation 
"Class :
   atomization
Direct Superclasses :
   context-item
Purpose :
   In conjonction with the recursion context item, provides a mechanism to obtain a new
   subcomposition list containing only the elementary subcompositions (leaves) in the
   same order as they appeared in the original hierarchy. For example, let the hierarchy
   (a (b (c d)) e (f (g))), the result of the realization will be (c d e g)." ) )
 
(defmethod realize-using-context-item ((context-item atomization) 
                                       (compo compo) 
                                       &key &allow-other-keys)

"Primary method
   realize-using-context-item specialized on atomization.
Behaviour
   The next method is first called.
   A new subcomposition list is given to the composition according to the following 
   rules :
   For each current subcomposition from compo :
   . If the subcomposition has not itself any subcomposition, it is kept as is in the 
     new subcomposition list.
   . If the subcomposition does have itself any subcompositions, the list of its 
     subcompositions is appended to the new subcomposition list."

  (declare (ignore context-item))
  (call-next-method)
  (with-slots (subs) compo
    (let ((subcomps))
      (dolist (elem subs)
        (setf subcomps 
              (append subcomps 
                      (or (subs elem)
                          (list elem) ) ) ) )
      (setf subs subcomps)) )
  compo )