;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defmacro mydeftype (name &rest args)

"Macro
   mydeftype (name &rest args)
Purpose
   mydeftype has the same behaviours and result that the Common Lisp standard macro
   'deftype' plus the side effect of associating the value t to the indicator
   'type-spec-p' in the type specifier symbol's property list."

  `(prog1
     (deftype ,name ,@args)
     (setf (get ',name 'type-spec-p) t) ) )

;; setting of the indicator 'type-spec-p' to t in any of the Common Lisp standard
;; type specifier symbols (those listed in the table page 50 in Steele : The 
;; Language - Second Edition).

(setf (get 'array 'type-spec-p) t)
(setf (get 'atom 'type-spec-p) t)
(setf (get 'bignum 'type-spec-p) t)
(setf (get 'bit 'type-spec-p) t)
(setf (get 'bit-vector 'type-spec-p) t)
(setf (get 'character 'type-spec-p) t)
(setf (get 'compiled-function 'type-spec-p) t)
(setf (get 'complex 'type-spec-p) t)
(setf (get 'cons 'type-spec-p) t)
(setf (get 'double-float 'type-spec-p) t)
(setf (get 'fixnum 'type-spec-p) t)
(setf (get 'float 'type-spec-p) t)
(setf (get 'function 'type-spec-p) t)
(setf (get 'hash-table 'type-spec-p) t)
(setf (get 'integer 'type-spec-p) t)
(setf (get 'keyword 'type-spec-p) t)
(setf (get 'list 'type-spec-p) t)
(setf (get 'long-float 'type-spec-p) t)
(setf (get 'nil 'type-spec-p) t)
(setf (get 'null 'type-spec-p) t)
(setf (get 'number 'type-spec-p) t)
(setf (get 'package 'type-spec-p) t)
(setf (get 'pathname 'type-spec-p) t)
(setf (get 'ramdom-state 'type-spec-p) t)
(setf (get 'ratio 'type-spec-p) t)
(setf (get 'rational 'type-spec-p) t)
(setf (get 'readtable 'type-spec-p) t)
(setf (get 'sequence 'type-spec-p) t)
(setf (get 'short-float 'type-spec-p) t)
(setf (get 'signed-byte 'type-spec-p) t)
(setf (get 'simple-array 'type-spec-p) t)
(setf (get 'simple-bit-vector 'type-spec-p) t)
(setf (get 'simple-string 'type-spec-p) t)
(setf (get 'simple-vector 'type-spec-p) t)
(setf (get 'single-float 'type-spec-p) t)
(setf (get 'standard-char 'type-spec-p) t)
(setf (get 'stream 'type-spec-p) t)
(setf (get 'string 'type-spec-p) t)
(setf (get 'symbol 'type-spec-p) t)
(setf (get 't 'type-spec-p) t)
(setf (get 'unsigned-byte 'type-spec-p) t)
(setf (get 'vector 'type-spec-p) t)
(setf (get 'member 'type-spec-p) t)
(setf (get 'eql 'type-spec-p) t)
(setf (get 'not 'type-spec-p) t)
(setf (get 'and 'type-spec-p) t)
(setf (get 'or 'type-spec-p) t)