;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defgeneric compo-lessp (x y)
  (:documentation

"Generic Function
   compo-lessp (x y)
Arguments
   x and y are two objects to be compared for inferiority.
Values
   x if x < y (in some appropriate sense to be defined in each primary method), nil
   otherwise.
Purpose
   This predicate is used in the sorting of compositions." ) )

(defmethod compo-lessp ((x abstract-compo) (y abstract-compo))

"Primary method
   compo-lessp specialized on 'abstract-compo'
Behaviour
   As a default, a composition x is less than a composition y if the Common Lisp
   predicate 'string-lessp' is true when applied to the names of x and y respectively,
   provided that the slot 'name' is not null for x and y."

  (and (name x) (name y) (string-lessp (name x) (name y))) )