;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defgeneric make-constant (compo context)
  (:documentation

"Generic Function
   make-constant (compo context)
Arguments
   An instance of 'abstract-compo' or one of its subclasses.
   An instance of one of 'context-item's subclasses.
Values
   A new object which shares the slots of compo, but which class is the constant class
   corresponding to it (as for example ccompo corresponds to abstract-compo).
Purpose
   This accessor is used by the generic function 'nrealize', to make the composition
   being realized a constant. The CLOS generic function 'change-class' was used first for
   the same purpose, but it has been replaced by 'make-constant' since 'change-class' is
   not implemented in CLisp." ) )

(defmethod make-constant ((compo abstract-compo)(context context-item))

"Primary method
   make-constant specialized on 'abstract-compo' and 'context-item'
Behaviour
   Returns an instance of the class 'ccompo' sharing the slots 'name', 'subs' and
   coercer from compo."

  (declare (ignore context))
  (make-instance 'ccompo
    :name (name compo)
    :subs (subs compo)
    :coercer (coercer compo) ) )