;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

;; specialized-print generic function

(defgeneric specialized-print (self stream sep)
  (:documentation

"Generic Function
   specialized-print (self stream sep)
Arguments
   A composition as a specializer. 
   The current output stream.
   A separator indicator.
Values
   A new separator indicator is returned.
   As a side effect, the composition is printed on the output stream." ) )

(defmethod specialized-print ((self abstract-compo) stream sep)

"Primary method
   specialized-print specialized on compo
Behaviour
   If the composition has not a name (its slot 'name' is nil), the function returns nil
   without doing anything.
   If the separator indicator is true, a space is printed first in the output stream.
   The composition name is printed in the output stream.
   t is returned as a result."

  (when (name self)
    (when sep (write-string " " stream))
    (write (name self) :stream stream)
    t ) )