;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(defun key->val (a f)

"Function
   key->val (a f)
Arguments
   An expression containing keywords to be converted to values.
   The lambda expression able to operates the conversion.
Values
   A copy of the expression where keywords are changed to values, dependind on the
   semantic of the converting lambda expression.
Purpose
   This function is used by the normalization process (see 'normalize-compo') to 
   converts keywords such as :C4 to 0 for example, in the case of the construction of
   notes (refer to the package :note).
Behaviour
   If the expression is an atom, returns whatever the conversion lambda expression
   returns when passing it this atom, except nil. If the conversion lambda expression
   returns nil, the atom is returned unchanged.
   If the expression is a cons, a new cons which car is the result of recursively
   applying key->val to the car of the expression, and which cdr is the result of
   recursively applying key->val to the cdr of the expression is returned."

  (if (atom a)
    (let ((p (funcall f a)))
      (if p p a) )
    (cons (key->val (car a) f) (key->val (cdr a) f)) ) )