;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :note
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :note)

(defparameter *note* "
*note* ::= ( *class-name* [*note-complement*] )" )

(defparameter *class-name* "
*class-name* ::= note | cnote | fermata | repeat-end | fermata-repeat-end | lyrics" )

(defparameter *note-complement* "
*note-complement* ::= [*comment*] {*parameters*}* {*sub-spec*}*}" )

(defparameter *parameters* "
*parameters* ::= *parameter-couple* | *parameter-key*" )

(defparameter *comment* (concatenate 'string "
*comment* ::= " *comment-descr* ))

(defparameter *parameter-couple* "
*parameter-couple* ::= *hei-couple* | *dur-couple* | *pos-mod-couple* | *pos-couple* | 
                       *voi-couple* | *dyn-couple* | *hei-coercer-couple* |
                       *dur-coercer-couple* | *pos-coercer-couple* |
                       *voi-coercer-couple* | *dyn-coercer-couple*" )

(defparameter *hei-couple* "
*hei-couple* ::= :hei *hei-value*" )

(defparameter *dur-couple* "
*dur-couple* ::= :dur *dur-value*" )

(defparameter *pos-mod-couple* "
*pos-mod-couple* ::= :pos-mod {*pos-mod-key*}" )

(defparameter *pos-couple* "
*pos-couple* ::= :pos *value*" )

(defparameter *voi-couple* "
*voi-couple* ::= :voi *voi-value*" )

(defparameter *dyn-couple* "
*dyn-couple* ::= :dyn *value*" )

(defparameter *hei-coercer-couple* "
*hei-coercer-couple* ::= :coerce-hei *hei-coercer*" )

(defparameter *hei-coercer* "
*hei-coercer* ::= '*coercer-specifier-name* | '( *hei-call* {*hei-value*}* )" )

(defparameter *hei-call* "
*hei-call* ::= *coercer-specifier-name* | ( *coercer-specifier-name* {*hei-value*}* )}" )

(defparameter *dur-coercer-couple* "
*dur-coercer-couple* ::= :coerce-dur *dur-coercer*" )

(defparameter *dur-coercer* "
*dur-coercer* ::= '*coercer-specifier-name* | '( *dur-call* {*dur-value*}* )" )

(defparameter *dur-call* "
*dur-call* ::= *coercer-specifier-name* | ( *coercer-specifier-name* {*dur-value*}* )}" )

(defparameter *pos-coercer-couple* "
*pos-coercer-couple* ::= :coerce-pos *pos-coercer*" )

(defparameter *pos-coercer* "
*pos-coercer* ::= '*coercer-specifier-name* | '( *call* {*value*}* )" )

(defparameter *voi-coercer-couple* "
*voi-coercer-couple* ::= :coerce-voi *voi-coercer*" )

(defparameter *voi-coercer* "
*voi-coercer* ::= '*coercer-specifier-name* | '( *voi-call* {*voi-value*}* )" )

(defparameter *voi-call* "
*voi-call* ::= *coercer-specifier-name* | ( *coercer-specifier-name* {*voi-value*}* )}" )

(defparameter *dyn-coercer-couple* "
*dyn-coercer-couple* ::= :coerce-dyn *dyn-coercer*" )

(defparameter *dyn-coercer* "
*dyn-coercer* ::= '*coercer-specifier-name* | '( *call* {*value*}* )" )

(defparameter *call* "
*call* ::= *coercer-specifier-name* | ( *coercer-specifier-name* {*value*}* )}" )

(defparameter *coercer-specifier-name* (concatenate 'string "
*coercer-specifier-name* ::= " *coercer-specifier-name-descr* ))

(defparameter *hei-value* "
*hei-value* ::= *hei-key* | *Common-Lisp-number* | *compo-lexical-variable* | *evaluable-hei* | nil" )

(defparameter *evaluable-hei* "
*evaluable-hei* ::= ( *hei-operator* {*hei-value*}* )" )

(defparameter *hei-operator* "
*hei-operator* ::= *operator-name* | ( *operator-name* {*hei-value*}* )" )

(defparameter *dur-value* "
*dur-value* ::= *dur-key* | *Common-Lisp-number* | *compo-lexical-variable* | *evaluable-dur*" )

(defparameter *evaluable-dur* "
*evaluable-dur* ::= ( *dur-operator* {*dur-value*}* )" )

(defparameter *dur-operator* "
*dur-operator* ::= *operator-name* | ( *operator-name* {*dur-value*}* )" )

(defparameter *voi-value* "
*voi-value* ::= *voi-key* | *Common-Lisp-number* | *compo-lexical-variable* | *evaluable-voi*" )

(defparameter *evaluable-voi* "
*evaluable-voi* ::= ( *voi-operator* {*voi-value*}* )" )

(defparameter *voi-operator* "
*voi-operator* ::= *operator-name* | ( *operator-name* {*voi-value*}* )" )

(defparameter *value* "
*value* ::= *Common-Lisp-number* | *compo-lexical-variable* | *evaluable*" )

(defparameter *evaluable* "
*evaluable* ::= ( *operator* {*value*}* )" )

(defparameter *operator* "
*operator* ::= *operator-name* | ( *operator-name* {*value*}* )" )

(defparameter *operator-name* (concatenate 'string "
*operator-name* ::= " *operator-name-descr* ))

(defparameter *common-lisp-number* (concatenate 'string "
*Common-Lisp-number* ::= " *common-lisp-number-descr* ))

(defparameter *compo-lexical-variable* "
*compo-lexical-variable* ::= super-hei | super-dur | super-pos | super-voi | super-dyn |
                             prev-hei | prev-dur | prev-pos | prev-voi | prev-dyn" )

(defparameter *parameter-key* "
*parameter-key* ::= *hei-key* | *dur-key* | *pos-mod-key* | *voi-key* | :rest | :chord" )

(defparameter *hei-key* "
*hei-key* ::= {*hei-elementary-key* [0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8]} |
               {:ac-bass-drum | :bs-drum | :side | :ac-snare | :clap | :el-snare |
                :lo-floor-tom | :cl-hh | :hi-floor-tom | :pedal-hh | :lo-tom |
                :op-hh | :lo-mid-tom | :hi-mid-tom | :crash | :hi-tom |
                :ride | :chinese | :ride-bell | :tambourine | :splash |
                :cowbel | :crash-2 | :vibraslap | :ride-2 | :hi-bongo |
                :lo-bongo | :mute-hi-conga | :op-hi-conga | :lo-conga |
                :hi-timb | :lo-timb | :hi-agogo | :lo-agogo | :cabasa |
                :maracas | :sh-whistle | :lg-whistle | :sh-guiro | :lg-guiro |
                :claves | :hi-wood-block | :lo-wood-block | :mute-cuica | :op-cuica |
                :mute-triangle | :op-triangle}" )

(defparameter *pos-mod-key* "
*pos-mod-key* ::= :abs | :juxt | :super" )

(defparameter *voi-key* "
*voi-key* ::= :french-violin | :treble | :soprano | :mezzo-soprano | :alto |
              :tenor | :baritone | :bass | :sub-bass | :double-bass | :percussion" )

(defparameter *sub-spec* "
*sub-spec* ::= *anonym* | *declaration* | *reference* | *multiple*" )

(defparameter *declaration* "
*declaration* ::= *symbol* = *anonym*" )

(defparameter *reference* "
*reference* ::= *symbol*" )

(defparameter *symbol* (concatenate 'string "
*symbol* ::= " *symbol-descr* ))

(defparameter *multiple* "
*multiple* ::= *integer* * *anonym*" )

(defparameter *anonym* "
*anonym* ::= *note* | *sub-note*" )

(defparameter *sub-note* "
*sub-note* ::= ( [*note-complement*] )" )

(defparameter *eng-hei-elementary-key* "
*hei-elementary-key* ::= :C | :D | :E | :F | :G | :A | :B |
                         :CS | :DS | :ES | :FS | :GS | :AS | :BS |
                         :CSS | :DSS | :ESS | :FSS | :GSS | :ASS | :BSS |
                         :CF | :DF | :EF | :FF | :GF | :AF | :BF |
                         :CFF | :DFF | :EFF | :FFF | :GFF | :AFF | :BFF" )

(defparameter *eng-dur-key* "
*dur-key* ::= :Q | :Q. | :Q.. | :H | :H. | :H.. |
              :W | :W. | :W.. | :DW | :DW. | :DW.. |
              :8th | :8th. | :8th.. | :16th | :16th. | :16th.. |
              :32th | :32th. | :32th.. | :64th | :64th. | :64th.." )

(defparameter *lat-hei-elementary-key* "
*hei-elementary-key* ::= :DO | :RE | :MI | :FA | :SOL | :LA | :SI |
                         :DO# | :RE# | :MI# | :FA# | :SOL# | :LA# | :SI# |
                         :DO## | :RE## | :MI## | :FA## | :SOL## | :LA## | :SI## |
                         :DOb | :REb | :MIb | :FAb | :SOLb | :LAb | :SIb |
                         :DObb | :REbb | :MIbb | :FAbb | :SOLbb | :LAbb | :SIbb |" )

(defparameter *lat-dur-key* "
*dur-key* ::= :N | :N. | :N.. | :B | :B. | :B.. |
             :R | :R. | :R.. | :DR | :DR. | :DR.. |
             :C | :C. | :C.. | :DC | :DC. | :DC.. |
             :TC | :TC. | :TC.. | :QC | :QC. | :QC.." )