;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :note
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :note)

(defconstant *english-pitches*

  '(:c0 :d0 :e0 :f0 :g0 :a0 :b0
    :cs0 :ds0 :es0 :fs0 :gs0 :as0 :bs0
    :cf0 :df0 :ef0 :ff0 :gf0 :af0 :bf0
    :css0 :dss0 :ess0 :fss0 :gss0 :ass0 :bss0
    :cff0 :dff0 :eff0 :fff0 :gff0 :aff0 :bff0
    
    :c1 :d1 :e1 :f1 :g1 :a1 :b1
    :cs1 :ds1 :es1 :fs1 :gs1 :as1 :bs1
    :cf1 :df1 :ef1 :ff1 :gf1 :af1 :bf1
    :css1 :dss1 :ess1 :fss1 :gss1 :ass1 :bss1
    :cff1 :dff1 :eff1 :fff1 :gff1 :aff1 :bff1
    
    :c2 :d2 :e2 :f2 :g2 :a2 :b2
    :cs2 :ds2 :es2 :fs2 :gs2 :as2 :bs2
    :cf2 :df2 :ef2 :ff2 :gf2 :af2 :bf2
    :css2 :dss2 :ess2 :fss2 :gss2 :ass2 :bss2
    :cff2 :dff2 :eff2 :fff2 :gff2 :aff2 :bff2
    
    :c3 :d3 :e3 :f3 :g3 :a3 :b3
    :cs3 :ds3 :es3 :fs3 :gs3 :as3 :bs3
    :cf3 :df3 :ef3 :ff3 :gf3 :af3 :bf3
    :css3 :dss3 :ess3 :fss3 :gss3 :ass3 :bss3
    :cff3 :dff3 :eff3 :fff3 :gff3 :aff3 :bff3
    
    :c4 :d4 :e4 :f4 :g4 :a4 :b4
    :cs4 :ds4 :es4 :fs4 :gs4 :as4 :bs4
    :cf4 :df4 :ef4 :ff4 :gf4 :af4 :bf4
    :css4 :dss4 :ess4 :fss4 :gss4 :ass4 :bss4
    :cff4 :dff4 :eff4 :fff4 :gff4 :aff4 :bff4
    
    :c5 :d5 :e5 :f5 :g5 :a5 :b5
    :cs5 :ds5 :es5 :fs5 :gs5 :as5 :bs5
    :cf5 :df5 :ef5 :ff5 :gf5 :af5 :bf5
    :css5 :dss5 :ess5 :fss5 :gss5 :ass5 :bss5
    :cff5 :dff5 :eff5 :fff5 :gff5 :aff5 :bff5
    
    :c6 :d6 :e6 :f6 :g6 :a6 :b6
    :cs6 :ds6 :es6 :fs6 :gs6 :as6 :bs6
    :cf6 :df6 :ef6 :ff6 :gf6 :af6 :bf6
    :css6 :dss6 :ess6 :fss6 :gss6 :ass6 :bss6
    :cff6 :dff6 :eff6 :fff6 :gff6 :aff6 :bff6
    
    :c7 :d7 :e7 :f7 :g7 :a7 :b7
    :cs7 :ds7 :es7 :fs7 :gs7 :as7 :bs7
    :cf7 :df7 :ef7 :ff7 :gf7 :af7 :bf7
    :css7 :dss7 :ess7 :fss7 :gss7 :ass7 :bss7
    :cff7 :dff7 :eff7 :fff7 :gff7 :aff7 :bff7
    
    :c8 :d8 :e8 :f8 :g8 :a8 :b8
    :cs8 :ds8 :es8 :fs8 :gs8 :as8 :bs8
    :cf8 :df8 :ef8 :ff8 :gf8 :af8 :bf8
    :css8 :dss8 :ess8 :fss8 :gss8 :ass8 :bss8
    :cff8 :dff8 :eff8 :fff8 :gff8 :aff8 :bff8 ) )

(defconstant *latin-pitches*

  '(:do0 :re0 :mi0 :fa0 :sol0 :la0 :si0
    :do#0 :re#0 :mi#0 :fa#0 :sol#0 :la#0 :si#0
    :dob0 :reb0 :mib0 :fab0 :solb0 :lab0 :sib0
    :do##0 :re##0 :mi##0 :fa##0 :sol##0 :la##0 :si##0
    :dobb0 :rebb0 :mibb0 :fabb0 :solbb0 :labb0 :sibb0
    
    :do1 :re1 :mi1 :fa1 :sol1 :la1 :si1
    :do#1 :re#1 :mi#1 :fa#1 :sol#1 :la#1 :si#1
    :dob1 :reb1 :mib1 :fab1 :solb1 :lab1 :sib1
    :do##1 :re##1 :mi##1 :fa##1 :sol##1 :la##1 :si##1
    :dobb1 :rebb1 :mibb1 :fabb1 :solbb1 :labb1 :sibb1
    
    :do2 :re2 :mi2 :fa2 :sol2 :la2 :si2
    :do#2 :re#2 :mi#2 :fa#2 :sol#2 :la#2 :si#2
    :dob2 :reb2 :mib2 :fab2 :solb2 :lab2 :sib2
    :do##2 :re##2 :mi##2 :fa##2 :sol##2 :la##2 :si##2
    :dobb2 :rebb2 :mibb2 :fabb2 :solbb2 :labb2 :sibb2
    
    :do3 :re3 :mi3 :fa3 :sol3 :la3 :si3
    :do#3 :re#3 :mi#3 :fa#3 :sol#3 :la#3 :si#3
    :dob3 :reb3 :mib3 :fab3 :solb3 :lab3 :sib3
    :do##3 :re##3 :mi##3 :fa##3 :sol##3 :la##3 :si##3
    :dobb3 :rebb3 :mibb3 :fabb3 :solbb3 :labb3 :sibb3
    
    :do4 :re4 :mi4 :fa4 :sol4 :la4 :si4
    :do#4 :re#4 :mi#4 :fa#4 :sol#4 :la#4 :si#4
    :dob4 :reb4 :mib4 :fab4 :solb4 :lab4 :sib4
    :do##4 :re##4 :mi##4 :fa##4 :sol##4 :la##4 :si##4
    :dobb4 :rebb4 :mibb4 :fabb4 :solbb4 :labb4 :sibb4
    
    :do5 :re5 :mi5 :fa5 :sol5 :la5 :si5
    :do#5 :re#5 :mi#5 :fa#5 :sol#5 :la#5 :si#5
    :dob5 :reb5 :mib5 :fab5 :solb5 :lab5 :sib5
    :do##5 :re##5 :mi##5 :fa##5 :sol##5 :la##5 :si##5
    :dobb5 :rebb5 :mibb5 :fabb5 :solbb5 :labb5 :sibb5
    
    :do6 :re6 :mi6 :fa6 :sol6 :la6 :si6
    :do#6 :re#6 :mi#6 :fa#6 :sol#6 :la#6 :si#6
    :dob6 :reb6 :mib6 :fab6 :solb6 :lab6 :sib6
    :do##6 :re##6 :mi##6 :fa##6 :sol##6 :la##6 :si##6
    :dobb6 :rebb6 :mibb6 :fabb6 :solbb6 :labb6 :sibb6
    
    :do7 :re7 :mi7 :fa7 :sol7 :la7 :si7
    :do#7 :re#7 :mi#7 :fa#7 :sol#7 :la#7 :si#7
    :dob7 :reb7 :mib7 :fab7 :solb7 :lab7 :sib7
    :do##7 :re##7 :mi##7 :fa##7 :sol##7 :la##7 :si##7
    :dobb7 :rebb7 :mibb7 :fabb7 :solbb7 :labb7 :sibb7
    
    :do8 :re8 :mi8 :fa8 :sol8 :la8 :si8
    :do#8 :re#8 :mi#8 :fa#8 :sol#8 :la#8 :si#8
    :dob8 :reb8 :mib8 :fab8 :solb8 :lab8 :sib8
    :do##8 :re##8 :mi##8 :fa##8 :sol##8 :la##8 :si##8
    :dobb8 :rebb8 :mibb8 :fabb8 :solbb8 :labb8 :sibb8 ) )

(eval-when (:execute :load-toplevel :compile-toplevel)
  (defun intern-temperament (l)

"Function
   intern-temperament (l)
Arguments
   A musical temperament to be interned. 
Values
   The required interned temperament.
Purpose
   This function allows to create new temperaments to be used by compo.
   The format expected for the argument is a list which first element is the octave value
   in cents (generally equal to 1200), and which rest is the value of each of the 35
   elementary pitches in the required order C, D, E, F, G, A, B, CS, DS, ES, FS, GS, AS,
   BS, CF, DF, EF, FF, GF, AF, BF, CSS, DSS, ESS, FSS, GSS, ASS, BSS, CFF, DFF, EFF,
   FFF, GFF, AFF and BFF. It is important that each pitch being included between 0
   (included) and the specified octave (excluded).
   The returned musical temperament, an association list managing a correspondance between
   pitches and values in cent, along the octaves from 0 to 8, is usable as a value to the
   compo parameter *default-temperament*, and as an argument to each pitch function
   accepting a temperament key argument (like 'itp' or 'pti' for instance)."

   (cons
     (car l)
     (pairlis *english-pitches*
              (mapcan #'(lambda (n)
                          (mapcar #'(lambda (a)
                                      (+ a (* n (car l))) )
                                  (cdr l) ) )
                      '(-4 -3 -2 -1 0 1 2 3 4) ) ) ) ) )

(let ((al-eng (pairlis
               '(:ac-bass-drum :bs-drum :side :ac-snare :clap :el-snare
                 :lo-floor-tom :cl-hh :hi-floor-tom :pedal-hh :lo-tom
                 :op-hh :lo-mid-tom :hi-mid-tom :crash :hi-tom
                 :ride :chinese :ride-bell :tambourine :splash
                 :cowbel :crash-2 :vibraslap :ride-2 :hi-bongo
                 :lo-bongo :mute-hi-conga :op-hi-conga :lo-conga
                 :hi-timb :lo-timb :hi-agogo :lo-agogo :cabasa
                 :maracas :sh-whistle :lg-whistle :sh-guiro :lg-guiro
                 :claves :hi-wood-block :lo-wood-block :mute-cuica :op-cuica
                 :mute-triangle :op-triangle

                 :c :d :e :f :g :a :b
                 :cs :ds :es :fs :gs :as :bs
                 :css :dss :ess :fss :gss :ass :bss
                 :cf :df :ef :ff :gf :af :bf
                 :cff :dff :eff :fff :gff :aff :bff )

               '(:b1
                 :c2 :cs2 :d2 :ef2 :e2 :f2 :fs2 :g2 :af2 :a2 :bf2 :b2
                 :c3 :cs3 :d3 :ef3 :e3 :f3 :fs3 :g3 :af3 :a3 :bf3 :b3
                 :c4 :cs4 :d4 :ef4 :e4 :f4 :fs4 :g4 :af4 :a4 :bf4 :b4
                 :c5 :cs5 :d5 :ef5 :e5 :f5 :fs5 :g5 :af5 :a5

                 :c4 :d4 :e4 :f4 :g4 :a4 :b4
                 :cs4 :ds4 :es4 :fs4 :gs4 :as4 :bs4
                 :css4 :dss4 :ess4 :fss4 :gss4 :ass4 :bss4
                 :cf4 :df4 :ef4 :ff4 :gf4 :af4 :bf4
                 :cff4 :dff4 :eff4 :fff4 :gff4 :aff4 :bff4 ) ) )

      (al-lat (pairlis
               '(:ac-bass-drum :bs-drum :side :ac-snare :clap :el-snare
                 :lo-floor-tom :cl-hh :hi-floor-tom :pedal-hh :lo-tom
                 :op-hh :lo-mid-tom :hi-mid-tom :crash :hi-tom
                 :ride :chinese :ride-bell :tambourine :splash
                 :cowbel :crash-2 :vibraslap :ride-2 :hi-bongo
                 :lo-bongo :mute-hi-conga :op-hi-conga :lo-conga
                 :hi-timb :lo-timb :hi-agogo :lo-agogo :cabasa
                 :maracas :sh-whistle :lg-whistle :sh-guiro :lg-guiro
                 :claves :hi-wood-block :lo-wood-block :mute-cuica :op-cuica
                 :mute-triangle :op-triangle

                 :do :re :mi :fa :sol :la :si
                 :do# :re# :mi# :fa# :sol# :la# :si#
                 :do## :re## :mi## :fa## :sol## :la## :si##
                 :dob :reb :mib :fab :solb :lab :sib
                 :dobb :rebb :mibb :fabb :solbb :labb :sibb )

               '(:si1
                 :do2 :do#2 :do#2 :mib2 :mi2 :fa2 :fa#2 :sol2 :lab2 :la2 :sib2 :si2
                 :do3 :do#3 :re3 :mib3 :mi3 :fa3 :fa#3 :sol3 :lab3 :la3 :sib3 :si3
                 :do4 :do#4 :re4 :mib4 :mi4 :fa4 :fa#4 :sol4 :lab4 :la4 :sib4 :si4
                 :do5 :do#5 :re5 :mib5 :mi5 :fa5 :fa#5 :sol5 :lab5 :la5

                 :do4 :re4 :mi4 :fa4 :sol4 :la4 :si4
                 :do#4 :re#4 :mi#4 :fa#4 :sol#4 :la#4 :si#4
                 :do##4 :re##4 :mi##4 :fa##4 :sol##4 :la##4 :si##4
                 :dob4 :reb4 :mib4 :fab4 :solb4 :lab4 :sib4
                 :dobb4 :rebb4 :mibb4 :fabb4 :solbb4 :labb4 :sibb4 ) ) ) )

  (defun normalize-pitch (pitch &key (syntax *default-syntax*))

"Function
   normalize-pitch (pitch &key :syntax)
Arguments
   A pitch to be normalized
   A syntax as a key argument, defaulting to the value of the special variable
   *default-syntax*.
Value
   The normalized pitch
Purpose
   Allows the using of non-octave-qualified pitches such as C, assuming in this case that
   the implicit octave is 4.
Behaviour
   Returns the pitch, making sure that it is qualified with its octave, and with regards
   to the specified or the default syntax. For example, in the syntax :english, C3 is
   returned as is while C is changed to C4. In the syntax :latin, RE is changed to RE4
   while MI1 remains MI1. The pitch is assumed to be valid in the specified or default
   syntax. For example, if RE is given to this function with the syntax :english, it is
   returned without changes, as it is the case for any invalid pitch in regards with the
   syntax.
   Note : This is for an optimization matter that the validity of that pitch being
   considered up to the caller, since this function is called by the predicate 'pitchp'
   wich already controls the pitch validity. If it is the case that this function being
   applied on a pitch that may be invalid, then 'pitchp' should be called instead, since
   it does return the octave-qualified pitch. nevertheless, 'normalize-pitch' is provided
   in addition to 'pitchp' in the case where a normalization is expected on a pitch which
   is already considered to be valid."

    (if (eq syntax :english)
      (or (cdr (assoc pitch al-eng)) pitch)            
      (or (cdr (assoc pitch al-lat)) pitch) ) )

  (defun denormalize-pitch (pitch &key (syntax *default-syntax*))

"Function
   denormalize-pitch (pitch &key :syntax)
Arguments
   A pitch to be denormalized
   A syntax as a key argument, defaulting to the value of the special variable
   *default-syntax*.
Value
   The denormalized pitch
Purpose
   Allows the presentation of non-octave-qualified pitches such as C, assuming in this
   case that the implicit octave is 4.
Behaviour
   This function is the counterpart of 'normalize-pitch'. It returns the pitch without
   its octave qualification if it is 4, with regard to the specified or default syntax."

    (if (eq syntax :english)
      (or (car (rassoc pitch al-eng)) pitch)            
      (or (car (rassoc pitch al-lat)) pitch) ) ) )

(let ((al (pairlis *latin-pitches* *english-pitches*)))

  (defun lat->eng (pitch)
    (cdr (assoc pitch al)) )

  (defun eng->lat (pitch)
    (car (rassoc pitch al)) ) )

(defun pitchp (x &key (syntax *default-syntax*))

"Predicate
   pitchp (x &key :syntax)
Arguments
   A value to be considered as being a pitch or not
   A syntax as a key argument, defaulting to the value of the special variable
   *default-syntax*
Value
   If the value is a pitch in the specified or default syntax, its normalization (see
   'normalize-pitch') is returned, otherwise the predicate returns false."

  (setf x (normalize-pitch x :syntax syntax))
  (and
   (member x (if (eq syntax :english) *english-pitches* *latin-pitches*))
   x ) )

(defun itp (interval &key (temperament *default-temperament*) (syntax *default-syntax*))

"Function
   itp (interval &key :temperament :syntax)
Arguments
   An interval to be converted to a pitch
   A temperament as a key argument, defaulting to the value of the special variable
   *default-temperament*
   A syntax as a key argument, defaulting to the value of the *default-syntax* special
   variable
Value
   If the interval expressed in cents belongs to those of the specified or default
   temperament, in the range of octave between 0 and 8, the corresponding pitch in the
   specified or default syntax is returned. Otherwise, nil is returned."

  ((lambda (p) (if (eq syntax :english) p (eng->lat p)))
   (car (rassoc interval (cdr temperament))) ) )

(defun itei (interval &key (temperament *default-temperament*))

"Function
   itei (interval &key :temperamen)
Arguments
   An interval to be converted to an elementary interval
   A temperament as a key argument, defaulting to the value of the special variable
   *default-temperament*
Values
   The pitch corresponding to the interval
   The octave to which the interval belongs
Behaviour
   If the interval is not a number, nil is returned.
   The quotient and the rest of the euclidian division of the interval by the octave held
   by the specified or default temperament are computed.
   The quotient + 4 and the rest are returned."

  (and 
   (numberp interval)
   (multiple-value-bind (oct pitch)(floor interval (car temperament))
     (values pitch (+ oct 4)) ) ) )

(defun itep (interval &key (temperament *default-temperament*) (syntax *default-syntax*))

"Function
   itep (interval &key :temperament :syntax)
Arguments
   An interval to be converted to an elementary pitch
   A temperament as a key argument, defaulting to the value of the special variable
   *default-temperament*
   A syntax as a key argument, defaulting to the value of the special variable
   *default-syntax*
Values
   The elementary pitch corresponding to the interval
   The octave to which the interval belongs
Purpose
   This function can be seen as a short cut to itei followed by itp.
Behaviour
   If the interval is not a number, nil is returned.
   The quotient and the rest of the euclidian division of the interval by the octave held
   by the specified or default temperament are computed.
   If the rest belongs to the intervals of the specified or default temperament, in the
   octave 4, the corresponding pitch in the specified or default syntax, and the quotient
   plus 4 are returned. Otherwise, nil is returned."

  (and 
   (numberp interval)
   (multiple-value-bind (oct pitch)(floor interval (car temperament))
     (let ((result (car (rassoc pitch (cdr temperament)))))
      (values (if (eq syntax :english) result (eng->lat result)) (+ oct 4)) ) ) ) )

(defun pti (pitch &key (temperament *default-temperament*) (syntax *default-syntax*) )

"Function
   pti (pitch &key :temperament :syntax)
Arguments
   A pitch to be converted to an interval
   A temperament as a key argument, defaulting to the value of the special variable
   *default-temperament*
   A syntax as a key argument, defaulting to the value of the special variable
   *default-syntax*
Value
   The interval corresponding to the pitch
Behaviour
   The pitch is normalized first (see 'normalize-pitch').
   If the specified or default syntax is not :english, the pitch is translated to the
   syntax :english.
   If the pitch belongs to those of the specified or default temperament, the
   corresponding interval is returned, otherwise nil is returned."

  (setf pitch (normalize-pitch pitch :syntax syntax))
  (cdr (assoc (if (eq syntax :english) pitch (lat->eng pitch))
              (cdr temperament))) )

(defun ptep (pitch &key (temperament *default-temperament*) (syntax *default-syntax*) )

"Function
   ptep (pitch &key :temperament :syntax)
Arguments
   A pitch to be converted to an elementary pitch
   A temperament as a key argument, defaulting to the value of the special variable
   *default-temperament*
   A syntax as a key argument, defaulting to the value of the special variable
   *default-syntax*
Value
   The elementary pitch corresponding to the pitch
   The octave to which the interval belongs
Behaviour
   'pti' is applied on the pitch, using the specified or default temperament and the
   specified or default syntax.
   If the result of pti is nil, it is returned.
   Else 'itep' is applied on the result of pti, using the specified or default temperament
   and the specified or default syntax.
   The result of 'itep' is returned."

  (let ((res (pti pitch :temperament temperament :syntax syntax)))
    (and res (itep res :temperament temperament :syntax syntax)) ) )

(defun naturalp (pitch &key (syntax *default-syntax*))

"Predicate
   naturalp (pitch &key :syntax)
Argument
   A value to be considered or not as a natural pitch
   A syntax as a key argument, defaulting to the value of the special variable
   *default-syntax*
Value
   The value if it is a natural pitch in the specified or default syntax, nil otherwise."

  (find pitch (if (eq syntax :english)
                '(:C0 :D0 :E0 :F0 :G0 :A0 :B0
                  :C1 :D1 :E1 :F1 :G1 :A1 :B1
                  :C2 :D2 :E2 :F2 :G2 :A2 :B2
                  :C3 :D3 :E3 :F3 :G3 :A3 :B3
                  :C4 :D4 :E4 :F4 :G4 :A4 :B4
                  :C5 :D5 :E5 :F5 :G5 :A5 :B5
                  :C6 :D6 :E6 :F6 :G6 :A6 :B6
                  :C7 :D7 :E7 :F7 :G7 :A7 :B7
                  :C8 :D8 :E8 :F8 :G8 :A8 :B8 )

                '(:Do0 :Re0 :Mi0 :Fa0 :Sol0 :La0 :Si0
                  :Do1 :Re1 :Mi1 :Fa1 :Sol1 :La1 :Si1
                  :Do2 :Re2 :Mi2 :Fa2 :Sol2 :La2 :Si2
                  :Do3 :Re3 :Mi3 :Fa3 :Sol3 :La3 :Si3
                  :Do4 :Re4 :Mi4 :Fa4 :Sol4 :La4 :Si4
                  :Do5 :Re5 :Mi5 :Fa5 :Sol5 :La5 :Si5
                  :Do6 :Re6 :Mi6 :Fa6 :Sol6 :La6 :Si6
                  :Do7 :Re7 :Mi7 :Fa7 :Sol7 :La7 :Si7
                  :Do8 :Re8 :Mi8 :Fa8 :Sol8 :La8 :Si8 ) ) ) )

(defun sharpp (pitch &key (syntax *default-syntax*))

"Predicate
   sharpp (pitch &key :syntax)
Argument
   A value to be considered or not as a one-sharp pitch
   A syntax as a key argument, defaulting to the value of the special variable
   *default-syntax*
Value
   The value if it is a one-sharp pitch in the specified or default syntax, nil otherwise."

  (find pitch (if (eq syntax :english)
                '(:CS0 :DS0 :ES0 :FS0 :GS0 :AS0 :BS0
                  :CS1 :DS1 :ES1 :FS1 :GS1 :AS1 :BS1
                  :CS2 :DS2 :ES2 :FS2 :GS2 :AS2 :BS2
                  :CS3 :DS3 :ES3 :FS3 :GS3 :AS3 :BS3
                  :CS4 :DS4 :ES4 :FS4 :GS4 :AS4 :BS4
                  :CS5 :DS5 :ES5 :FS5 :GS5 :AS5 :BS5
                  :CS6 :DS6 :ES6 :FS6 :GS6 :AS6 :BS6
                  :CS7 :DS7 :ES7 :FS7 :GS7 :AS7 :BS7
                  :CS8 :DS8 :ES8 :FS8 :GS8 :AS8 :BS8 )

                '(:Do#0 :Re#0 :Mi#0 :Fa#0 :Sol#0 :La#0 :Si#0
                  :Do#1 :Re#1 :Mi#1 :Fa#1 :Sol#1 :La#1 :Si#1
                  :Do#2 :Re#2 :Mi#2 :Fa#2 :Sol#2 :La#2 :Si#2
                  :Do#3 :Re#3 :Mi#3 :Fa#3 :Sol#3 :La#3 :Si#3
                  :Do#4 :Re#4 :Mi#4 :Fa#4 :Sol#4 :La#4 :Si#4
                  :Do#5 :Re#5 :Mi#5 :Fa#5 :Sol#5 :La#5 :Si#5
                  :Do#6 :Re#6 :Mi#6 :Fa#6 :Sol#6 :La#6 :Si#6
                  :Do#7 :Re#7 :Mi#7 :Fa#7 :Sol#7 :La#7 :Si#7
                  :Do#8 :Re#8 :Mi#8 :Fa#8 :Sol#8 :La#8 :Si#8 ) ) ) )

(defun flatp (pitch &key (syntax *default-syntax*))

"Predicate
   flatp (pitch &key :syntax)
Argument
   A value to be considered or not as a one-flat pitch
   A syntax as a key argument, defaulting to the value of the special variable
   *default-syntax*
Value
   The value if it is a one-flat pitch in the specified or default syntax, nil otherwise."

  (find pitch (if (eq syntax :english)
                '(:CF0 :DF0 :EF0 :FF0 :GF0 :AF0 :BF0
                  :CF1 :DF1 :EF1 :FF1 :GF1 :AF1 :BF1
                  :CF2 :DF2 :EF2 :FF2 :GF2 :AF2 :BF2
                  :CF3 :DF3 :EF3 :FF3 :GF3 :AF3 :BF3
                  :CF4 :DF4 :EF4 :FF4 :GF4 :AF4 :BF4
                  :CF5 :DF5 :EF5 :FF5 :GF5 :AF5 :BF5
                  :CF6 :DF6 :EF6 :FF6 :GF6 :AF6 :BF6
                  :CF7 :DF7 :EF7 :FF7 :GF7 :AF7 :BF7
                  :CF8 :DF8 :EF8 :FF8 :GF8 :AF8 :BF8 )

                '(:Dob0 :Reb0 :Mib0 :Fab0 :Solb0 :Lab0 :Sib0
                  :Dob1 :Reb1 :Mib1 :Fab1 :Solb1 :Lab1 :Sib1
                  :Dob2 :Reb2 :Mib2 :Fab2 :Solb2 :Lab2 :Sib2
                  :Dob3 :Reb3 :Mib3 :Fab3 :Solb3 :Lab3 :Sib3
                  :Dob4 :Reb4 :Mib4 :Fab4 :Solb4 :Lab4 :Sib4
                  :Dob5 :Reb5 :Mib5 :Fab5 :Solb5 :Lab5 :Sib5
                  :Dob6 :Reb6 :Mib6 :Fab6 :Solb6 :Lab6 :Sib6
                  :Dob7 :Reb7 :Mib7 :Fab7 :Solb7 :Lab7 :Sib7
                  :Dob8 :Reb8 :Mib8 :Fab8 :Solb8 :Lab8 :Sib8 ) ) ) )

(defun double-sharp-p (pitch &key (syntax *default-syntax*))

"Predicate
   double-sharp-p (pitch &key :syntax)
Argument
   A value to be considered or not as a double-sharp pitch
   A syntax as a key argument, defaulting to the value of the special variable
   *default-syntax*
Value
   The value if it is a double-sharp pitch in the specified or default syntax, nil
   otherwise."

  (find pitch (if (eq syntax :english)
                '(:CSS0 :DSS0 :ESS0 :FSS0 :GSS0 :ASS0 :BSS0
                  :CSS1 :DSS1 :ESS1 :FSS1 :GSS1 :ASS1 :BSS1
                  :CSS2 :DSS2 :ESS2 :FSS2 :GSS2 :ASS2 :BSS2
                  :CSS3 :DSS3 :ESS3 :FSS3 :GSS3 :ASS3 :BSS3
                  :CSS4 :DSS4 :ESS4 :FSS4 :GSS4 :ASS4 :BSS4
                  :CSS5 :DSS5 :ESS5 :FSS5 :GSS5 :ASS5 :BSS5
                  :CSS6 :DSS6 :ESS6 :FSS6 :GSS6 :ASS6 :BSS6
                  :CSS7 :DSS7 :ESS7 :FSS7 :GSS7 :ASS7 :BSS7
                  :CSS8 :DSS8 :ESS8 :FSS8 :GSS8 :ASS8 :BSS8 )

                '(:Do##0 :Re##0 :Mi##0 :Fa##0 :Sol##0 :La##0 :Si##0
                  :Do##1 :Re##1 :Mi##1 :Fa##1 :Sol##1 :La##1 :Si##1
                  :Do##2 :Re##2 :Mi##2 :Fa##2 :Sol##2 :La##2 :Si##2
                  :Do##3 :Re##3 :Mi##3 :Fa##3 :Sol##3 :La##3 :Si##3
                  :Do##4 :Re##4 :Mi##4 :Fa##4 :Sol##4 :La##4 :Si##4
                  :Do##5 :Re##5 :Mi##5 :Fa##5 :Sol##5 :La##5 :Si##5
                  :Do##6 :Re##6 :Mi##6 :Fa##6 :Sol##6 :La##6 :Si##6
                  :Do##7 :Re##7 :Mi##7 :Fa##7 :Sol##7 :La##7 :Si##7
                  :Do##8 :Re##8 :Mi##8 :Fa##8 :Sol##8 :La##8 :Si##8 ) ) ) )

(defun double-flat-p (pitch &key (syntax *default-syntax*))

"Predicate
   double-flat-p (pitch &key :syntax)
Argument
   A value to be considered or not as a double-flat pitch
   A syntax as a key argument, defaulting to the value of the special variable
   *default-syntax*
Value
   The value if it is a double-flat pitch in the specified or default syntax, nil
   otherwise."

  (find pitch (if (eq syntax :english)
                '(:CFF0 :DFF0 :EFF0 :FFF0 :GFF0 :AFF0 :BFF0
                  :CFF1 :DFF1 :EFF1 :FFF1 :GFF1 :AFF1 :BFF1
                  :CFF2 :DFF2 :EFF2 :FFF2 :GFF2 :AFF2 :BFF2
                  :CFF3 :DFF3 :EFF3 :FFF3 :GFF3 :AFF3 :BFF3
                  :CFF4 :DFF4 :EFF4 :FFF4 :GFF4 :AFF4 :BFF4
                  :CFF5 :DFF5 :EFF5 :FFF5 :GFF5 :AFF5 :BFF5
                  :CFF6 :DFF6 :EFF6 :FFF6 :GFF6 :AFF6 :BFF6
                  :CFF7 :DFF7 :EFF7 :FFF7 :GFF7 :AFF7 :BFF7
                  :CFF8 :DFF8 :EFF8 :FFF8 :GFF8 :AFF8 :BFF8 )

                '(:Dobb0 :Rebb0 :Mibb0 :Fabb0 :Solbb0 :Labb0 :Sibb0
                  :Dobb1 :Rebb1 :Mibb1 :Fabb1 :Solbb1 :Labb1 :Sibb1
                  :Dobb2 :Rebb2 :Mibb2 :Fabb2 :Solbb2 :Labb2 :Sibb2
                  :Dobb3 :Rebb3 :Mibb3 :Fabb3 :Solbb3 :Labb3 :Sibb3
                  :Dobb4 :Rebb4 :Mibb4 :Fabb4 :Solbb4 :Labb4 :Sibb4
                  :Dobb5 :Rebb5 :Mibb5 :Fabb5 :Solbb5 :Labb5 :Sibb5
                  :Dobb6 :Rebb6 :Mibb6 :Fabb6 :Solbb6 :Labb6 :Sibb6
                  :Dobb7 :Rebb7 :Mibb7 :Fabb7 :Solbb7 :Labb7 :Sibb7
                  :Dobb8 :Rebb8 :Mibb8 :Fabb8 :Solbb8 :Labb8 :Sibb8 ) ) ) )

(defun funcptf (temperament pitch-fork)

"Function
   funcptf (temperament pitch-fork)
Arguments
   A temperament (see below)
   A pitch-fork giving the frequency in herz of A4
Value
   A function taking a pitch and a :syntax key argument (defaulting to the value of the
   special variable *default-syntax*), and converting the pitch expressed in the specified
   or default syntax to its corresponding frequency in the captured temperament and pitch
   fork.
Behaviour
   The temperament is expected to be in the form of an association list between elementary
   pitches in the normalized (octave-qualified) syntax :english and frequency values. The
   last item of the association list should be the pair (:octave . freq) where freq is the
   frequency ratio of the octave (generally 2).
   Behaviour of the produced function :
   1) If the specified or default syntax is not the syntax :english, the pitch argument is
      translated to the syntax :english;
   2) The elementary pitch and the octave number corresponding to the pitch are obtained
      by calling 'ptep' with the pitch and the syntax :english;
   3) The result of the function is computed according to the formula :
      r = ef * (octf power (oct - 4)) * pf / a4f
   Where :
   . ef is the frequency corresponding to the elementary pitch obtained in 2, in the
     captured temperament;
   . octf is the frequency corresponding to the keyword :octave in the captured
     temperament;
     oct is the octave number obtained in 2;
   . pf is the captured pitch-fork in hz;
   . a4f is the frequency corresponding to the A4 pitch in the captured temperament."

  #'(lambda (pitch &key (syntax *default-syntax*))
      (when (not (eq syntax :english))
        (setf pitch (lat->eng pitch)) )
      (multiple-value-bind (ep oct) (ptep pitch :syntax :english)
        (/
         (* (cdr (assoc ep temperament)) 
            (expt (cdr (assoc :octave temperament)) 
                  (- oct 4)) 
            pitch-fork)
         (cdr (assoc :A4 temperament)) ) ) ) )

(let ((pythagorician-temperament 
       (pairlis
        (list :octave
              :CFF4 :CF4 :C4 :CS4 :CSS4 
              :DFF4 :DF4 :D4 :DS4 :DSS4 
              :EFF4 :EF4 :E4 :ES4 :ESS4
              :FFF4 :FF4 :F4 :FS4 :FSS4 
              :GFF4 :GF4 :G4 :GS4 :GSS4 
              :AFF4 :AF4 :A4 :AS4 :ASS4
              :BFF4 :BF4 :B4 :BS4 :BSS4)
        (list 2
              (/ 16 9) (/ 243 128) 1 (/ 256 243) (/ 9 8)
              1 (/ 256 243) (/ 9 8) (/ 32 27) (/ 81 64)
              (/ 9 8) (/ 32 27) (/ 81 64) (/ 4 3) (/ 729 512)
              (/ 32 27) (/ 81 64) (/ 4 3) (/ 729 512) (/ 3 2)
              (/ 4 3) (/ 729 512) (/ 3 2) (/ 128 81) (/ 27 16)
              (/ 3 2) (/ 128 81) (/ 27 16) (/ 16 9) (/ 243 128)
              (/ 27 16) (/ 16 9) (/ 243 128) 1 (/ 256 243) ) ) )

      (equal-temperament
       (pairlis
        (list :octave
              :CFF4 :CF4 :C4 :CS4 :CSS4 
              :DFF4 :DF4 :D4 :DS4 :DSS4
              :EFF4 :EF4 :E4 :ES4 :ESS4
              :FFF4 :FF4 :F4 :FS4 :FSS4
              :GFF4 :GF4 :G4 :GS4 :GSS4
              :AFF4 :AF4 :A4 :AS4 :ASS4
              :BFF4 :BF4 :B4 :BS4 :BSS4)
        (list 2
              (expt 2 (/ 5 6)) (expt 2 (/ 11 12)) 1 (expt 2 (/ 1 12)) (expt 2 (/ 1 6))
              1 (expt 2 (/ 1 12)) (expt 2 (/ 1 6)) (expt 2 (/ 1 4)) (expt 2 (/ 1 3))
              (expt 2 (/ 1 6))  (expt 2 (/ 1 4)) (expt 2 (/ 1 3)) (expt 2 (/ 5 12)) (expt 2 (/ 1 2))
              (expt 2 (/ 1 4)) (expt 2 (/ 1 3)) (expt 2 (/ 5 12)) (expt 2 (/ 1 2)) (expt 2 (/ 7 12))
              (expt 2 (/ 5 12)) (expt 2 (/ 1 2)) (expt 2 (/ 7 12)) (expt 2 (/ 2 3)) (expt 2 (/ 3 4))
              (expt 2 (/ 7 12)) (expt 2 (/ 2 3)) (expt 2 (/ 3 4)) (expt 2 (/ 5 6)) (expt 2 (/ 11 12))
              (expt 2 (/ 3 4)) (expt 2 (/ 5 6)) (expt 2 (/ 11 12)) 1 (expt 2 (/ 1 12)) ) ) ) )

  (setf (symbol-function 'ptf-eq-440) (funcptf equal-temperament 440))
  (setf (documentation 'ptf-eq-440 'function)

"Function
   ptf-eq-440 (pitch &key :syntax)
Arguments
   A pitch
   A syntax as a key argument, defaulting to the value of the *default-syntax* special
   variable
Value
   The frequency value corresponding to the pitch expressed in the specified or default
   syntax
Behaviour
   This function has been produced by calling the function constructor 'funcptf', with the
   equal temperament and a A4 set to 440 hz as a pitch fork. See 'funcptf'." )

  (setf (symbol-function 'ptf-py-440) (funcptf pythagorician-temperament 440))
  (setf (documentation 'ptf-py-440 'function)
"Function
   ptf-py-440 (pitch &key :syntax)
Arguments
   A pitch
   A syntax as a key argument, defaulting to the value of the *default-syntax* special
   variable
Value
   The frequency value corresponding to the pitch expressed in the specified or default
   syntax
Behaviour
   This function has been produced by calling the function constructor 'funcptf', with the
   pythagorician temperament and a A4 set to 440 hz as a pitch fork. See 'funcptf'." ) )