;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :note
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :note)

;(sugar-compo rel-context rel-context)

(defmacro sugar-note (macro-name class-name &key (temperament '*default-temperament*)
                                                 name
                                                 (hei :C4)
                                                 (voi :treble)
                                                 (pos-mod :juxt)
                                                 (pos 0)
                                                 (dur :q)
                                                 (dyn 1)
                                                 subs )
"Macro
   sugar-note (macro-name class-name &key (temperament '*default-temperament*)
                                           name
                                           (hei :C4)
                                           (voi :treble)
                                           (pos-mod :juxt)
                                           (pos 0)
                                           (dur :q)
                                           (dyn 1)
                                           subs )
Values
   Returns a new macro named by the given name and able to construct new notes of the
   given class, with the given name, numeric property values and subs as default
   key-arguments.
Purpose
   This macro can be considered as the <<reader>> of compo for notes (think of the term
   <<reader>> in the same meaning as in Lisp). It allows the construction of hierarchies
   of notes in a more natural syntax than using make-instance. For example, one can
   write :
     (note :D4 :dyn 1/2 ()(:E4)())
   rather than :
     (make-instance 'note
       :hei :D4
       :dyn 1/2
       :subs (list (make-instance 'note)
                   (make-instance 'note :hei :E4)
                   (make-instance 'note) ) )
   When compo is loaded, sugar-note is applied to :
   . 'note' with the name 'note' and a default position mode of :juxt;
   . 'fermata' with the name 'fermata' and a default position mode of :juxt;
   . 'cnote' with the name 'cnote'.
 So at the begining of a compo session, one can safely enter :
     (note ()()())
     (cnote)
     ..."

  `(progn
     (setf (get ',class-name 'compo::compo-subclass) t)
     (defmacro ,macro-name (&rest restargs)
       (append (list 'make-instance '',class-name)
               (read-compo
                (apply
                 #'normalize-compo
                 restargs ',class-name (make-instance ',class-name)
                 '(:name ,name
                   :hei ,(key->val (quote-expr hei)
                                   #'(lambda (a)
                                       (pti (normalize-pitch a)
                                            :temperament (eval temperament)) ) )
                   :dur ,(key->val (quote-expr dur) #'(lambda (a) (dton a)))
                   :pos-mod ,pos-mod
                   :pos ,(quote-expr pos)
                   :voi ,(key->val voi #'(lambda (a) (cton a)))
                   :dyn ,(quote-expr dyn) )
                 (and ,temperament (list :temperament ,temperament)) ) )
               (and ,name (list :name ,name))
               ((lambda (a) (and a (list :hei (clone a))))
                ',(key->val (quote-expr hei)
                            #'(lambda (a)
                                (pti (normalize-pitch a)
                                     :temperament (eval temperament) ) ) ) )
               ((lambda (a) (and a (list :dur (clone a))))
                ',(key->val (quote-expr dur) #'(lambda (a) (dton a))) )
               (and ,pos-mod (list :pos-mod ,pos-mod))
               ((lambda (a) (and a (list :pos (clone a)))) ',(quote-expr pos) )
               ((lambda (a) (and a (list :voi (clone a))))
                ',(key->val (quote-expr voi) #'(lambda (a) (cton a))) )
               ((lambda (a) (and a (list :dyn (clone a)))) ',(quote-expr dyn))
               (and ,subs (list :subs (clone ,subs))) ) ) ) )

(sugar-note note note :pos-mod :juxt)
(sugar-note fermata fermata :pos-mod :juxt)
(sugar-note repeat-end repeat-end :pos-mod :juxt)
(sugar-note fermata-repeat-end fermata-repeat-end :pos-mod :juxt)
(sugar-note lyrics lyrics :pos-mod :juxt)
(sugar-note cnote cnote)